@@ -16,9 +16,11 @@
 import uuid
 
 from oslo_log import log as logging
+from oslo_utils import strutils
 from oslo_utils import timeutils
 import pecan
 import six
+import warnings
 import wsme
 from wsme import types as wtypes
 
@@ -103,7 +105,7 @@ class Cluster(base.APIBase):
                           default=None)
     """The name of the nova ssh keypair"""
 
-    node_count = wsme.wsattr(wtypes.IntegerType(minimum=1), default=1)
+    node_count = wsme.wsattr(wtypes.IntegerType(minimum=0), default=1)
     """The node count for this cluster. Default to 1 if not set"""
 
     master_count = wsme.wsattr(wtypes.IntegerType(minimum=1), default=1)
@@ -331,6 +333,12 @@ class ClustersController(base.Controller):
         'detail': ['GET'],
     }
 
+    _in_tree_cinder_volume_driver_deprecation_note = (
+        "The in-tree Cinder volume driver is deprecated and will be removed "
+        "in X cycle in favour of out-of-tree Cinder CSI driver which requires "
+        "the label cinder_csi_enabled set to True (default behaviour from "
+        "V cycle) when volume_driver is cinder.")
+
     actions = cluster_actions.ActionsController()
 
     def _generate_name_for_cluster(self, context):
@@ -482,10 +490,23 @@ class ClustersController(base.Controller):
                     "one.") % cluster_limit
             raise exception.ResourceLimitExceeded(msg=msg)
 
+    @base.Controller.api_version("1.1", "1.9")
     @expose.expose(ClusterID, body=Cluster, status_code=202)
     @validation.enforce_cluster_type_supported()
     @validation.enforce_cluster_volume_storage_size()
     def post(self, cluster):
+        if cluster.node_count == 0:
+            raise exception.ZeroNodeCountNotSupported()
+        return self._post(cluster)
+
+    @base.Controller.api_version("1.10")  # noqa
+    @expose.expose(ClusterID, body=Cluster, status_code=202)
+    @validation.enforce_cluster_type_supported()
+    @validation.enforce_cluster_volume_storage_size()
+    def post(self, cluster):  # noqa
+        return self._post(cluster)
+
+    def _post(self, cluster):
         """Create a new cluster.
 
         :param cluster: a cluster within the request body.
@@ -504,7 +525,7 @@ class ClustersController(base.Controller):
             cluster.keypair = cluster_template.keypair_id
 
         # If labels is not present, use cluster_template value
-        if cluster.labels == wtypes.Unset:
+        if cluster.labels == wtypes.Unset or not cluster.labels:
             cluster.labels = cluster_template.labels
         else:
             # If labels are provided check if the user wishes to merge
@@ -514,6 +535,13 @@ class ClustersController(base.Controller):
                 labels.update(cluster.labels)
                 cluster.labels = labels
 
+        cinder_csi_enabled = cluster.labels.get('cinder_csi_enabled', True)
+        if (cluster_template.volume_driver == 'cinder' and
+                not strutils.bool_from_string(cinder_csi_enabled)):
+            warnings.warn(self._in_tree_cinder_volume_driver_deprecation_note,
+                          DeprecationWarning)
+            LOG.warning(self._in_tree_cinder_volume_driver_deprecation_note)
+
         # If floating_ip_enabled is not present, use cluster_template value
         if cluster.floating_ip_enabled == wtypes.Unset:
             cluster.floating_ip_enabled = cluster_template.floating_ip_enabled
@@ -570,16 +598,40 @@ class ClustersController(base.Controller):
         (cluster, node_count,
          health_status,
          health_status_reason) = self._patch(cluster_ident, patch)
+        if node_count == 0:
+            raise exception.ZeroNodeCountNotSupported()
         pecan.request.rpcapi.cluster_update_async(cluster, node_count,
                                                   health_status,
                                                   health_status_reason)
         return ClusterID(cluster.uuid)
 
-    @base.Controller.api_version("1.3")  # noqa
+    @base.Controller.api_version("1.3", "1.9")  # noqa
     @wsme.validate(types.uuid, bool, [ClusterPatchType])
     @expose.expose(ClusterID, types.uuid_or_name, types.boolean,
                    body=[ClusterPatchType], status_code=202)
-    def patch(self, cluster_ident, rollback=False, patch=None):
+    def patch(self, cluster_ident, rollback=False, patch=None):  # noqa
+        """Update an existing Cluster.
+
+        :param cluster_ident: UUID or logical name of a cluster.
+        :param rollback: whether to rollback cluster on update failure.
+        :param patch: a json PATCH document to apply to this cluster.
+        """
+        (cluster, node_count,
+         health_status,
+         health_status_reason) = self._patch(cluster_ident, patch)
+        if node_count == 0:
+            raise exception.ZeroNodeCountNotSupported()
+        pecan.request.rpcapi.cluster_update_async(cluster, node_count,
+                                                  health_status,
+                                                  health_status_reason,
+                                                  rollback)
+        return ClusterID(cluster.uuid)
+
+    @base.Controller.api_version("1.10")  # noqa
+    @wsme.validate(types.uuid, bool, [ClusterPatchType])
+    @expose.expose(ClusterID, types.uuid_or_name, types.boolean,
+                   body=[ClusterPatchType], status_code=202)
+    def patch(self, cluster_ident, rollback=False, patch=None):  # noqa
         """Update an existing Cluster.
 
         :param cluster_ident: UUID or logical name of a cluster.
