@@ -151,9 +151,9 @@ class Connection(api.Connection):
         if filters is None:
             filters = {}
 
-        possible_filters = ["cluster_template_id", "name", "node_count",
-                            "master_count", "stack_id", "api_address",
-                            "node_addresses", "project_id", "user_id"]
+        possible_filters = ["cluster_template_id", "name", "stack_id",
+                            "api_address", "node_addresses", "project_id",
+                            "user_id"]
 
         filter_names = set(filters).intersection(possible_filters)
         filter_dict = {filter_name: filters[filter_name]
@@ -164,6 +164,26 @@ class Connection(api.Connection):
         if 'status' in filters:
             query = query.filter(models.Cluster.status.in_(filters['status']))
 
+        # Helper to filter based on node_count field from nodegroups
+        def filter_node_count(query, node_count, is_master=False):
+            nfunc = func.sum(models.NodeGroup.node_count)
+            nquery = model_query(models.NodeGroup)
+            if is_master:
+                nquery = nquery.filter(models.NodeGroup.role == 'master')
+            else:
+                nquery = nquery.filter(models.NodeGroup.role != 'master')
+            nquery = nquery.group_by(models.NodeGroup.cluster_id)
+            nquery = nquery.having(nfunc == node_count)
+            uuids = [ng.cluster_id for ng in nquery.all()]
+            return query.filter(models.Cluster.uuid.in_(uuids))
+
+        if 'node_count' in filters:
+            query = filter_node_count(
+                query, filters['node_count'], is_master=False)
+        if 'master_count' in filters:
+            query = filter_node_count(
+                query, filters['master_count'], is_master=True)
+
         return query
 
     def get_cluster_list(self, context, filters=None, limit=None, marker=None,
@@ -219,9 +239,8 @@ class Connection(api.Connection):
 
     def get_cluster_stats(self, context, project_id=None):
         query = model_query(models.Cluster)
-        node_count_col = models.Cluster.node_count
-        master_count_col = models.Cluster.master_count
-        ncfunc = func.sum(node_count_col + master_count_col)
+        node_count_col = models.NodeGroup.node_count
+        ncfunc = func.sum(node_count_col)
 
         if project_id:
             query = query.filter_by(project_id=project_id)
@@ -370,7 +389,8 @@ class Connection(api.Connection):
     def _is_publishing_cluster_template(self, values):
         if (len(values) == 1 and (
                 ('public' in values and values['public'] is True) or
-                ('hidden' in values))):
+                ('hidden' in values) or
+                ('tags' in values and values['tags'] is not None))):
             return True
         return False
 
@@ -762,3 +782,119 @@ class Connection(api.Connection):
             ref.update(values)
 
         return ref
+
+    def _add_nodegoup_filters(self, query, filters):
+        if filters is None:
+            filters = {}
+
+        possible_filters = ["name", "node_count", "node_addresses",
+                            "role", "is_default"]
+
+        filter_names = set(filters).intersection(possible_filters)
+        filter_dict = {filter_name: filters[filter_name]
+                       for filter_name in filter_names}
+
+        query = query.filter_by(**filter_dict)
+
+        if 'status' in filters:
+            query = query.filter(
+                models.NodeGroup.status.in_(filters['status']))
+
+        return query
+
+    def create_nodegroup(self, values):
+        if not values.get('uuid'):
+            values['uuid'] = uuidutils.generate_uuid()
+
+        nodegroup = models.NodeGroup()
+        nodegroup.update(values)
+        try:
+            nodegroup.save()
+        except db_exc.DBDuplicateEntry:
+            raise exception.NodeGroupAlreadyExists(
+                cluster_id=values['cluster_id'], name=values['name'])
+        return nodegroup
+
+    def destroy_nodegroup(self, cluster_id, nodegroup_id):
+        session = get_session()
+        with session.begin():
+            query = model_query(models.NodeGroup, session=session)
+            query = add_identity_filter(query, nodegroup_id)
+            query = query.filter_by(cluster_id=cluster_id)
+            try:
+                query.one()
+            except NoResultFound:
+                raise exception.NodeGroupNotFound(nodegroup=nodegroup_id)
+            query.delete()
+
+    def update_nodegroup(self, cluster_id, nodegroup_id, values):
+        return self._do_update_nodegroup(cluster_id, nodegroup_id, values)
+
+    def _do_update_nodegroup(self, cluster_id, nodegroup_id, values):
+        session = get_session()
+        with session.begin():
+            query = model_query(models.NodeGroup, session=session)
+            query = add_identity_filter(query, nodegroup_id)
+            query = query.filter_by(cluster_id=cluster_id)
+            try:
+                ref = query.with_lockmode('update').one()
+            except NoResultFound:
+                raise exception.NodeGroupNotFound(nodegroup=nodegroup_id)
+
+            ref.update(values)
+        return ref
+
+    def get_nodegroup_by_id(self, context, cluster_id, nodegroup_id):
+        query = model_query(models.NodeGroup)
+        if not context.is_admin:
+            query = query.filter_by(project_id=context.project_id)
+        query = query.filter_by(cluster_id=cluster_id)
+        query = query.filter_by(id=nodegroup_id)
+        try:
+            return query.one()
+        except NoResultFound:
+            raise exception.NodeGroupNotFound(nodegroup=nodegroup_id)
+
+    def get_nodegroup_by_uuid(self, context, cluster_id, nodegroup_uuid):
+        query = model_query(models.NodeGroup)
+        if not context.is_admin:
+            query = query.filter_by(project_id=context.project_id)
+        query = query.filter_by(cluster_id=cluster_id)
+        query = query.filter_by(uuid=nodegroup_uuid)
+        try:
+            return query.one()
+        except NoResultFound:
+            raise exception.NodeGroupNotFound(nodegroup=nodegroup_uuid)
+
+    def get_nodegroup_by_name(self, context, cluster_id, nodegroup_name):
+        query = model_query(models.NodeGroup)
+        if not context.is_admin:
+            query = query.filter_by(project_id=context.project_id)
+        query = query.filter_by(cluster_id=cluster_id)
+        query = query.filter_by(name=nodegroup_name)
+        try:
+            return query.one()
+        except MultipleResultsFound:
+            raise exception.Conflict('Multiple nodegroups exist with same '
+                                     'name. Please use the nodegroup uuid '
+                                     'instead.')
+        except NoResultFound:
+            raise exception.NodeGroupNotFound(nodegroup=nodegroup_name)
+
+    def list_cluster_nodegroups(self, context, cluster_id, filters=None,
+                                limit=None, marker=None, sort_key=None,
+                                sort_dir=None):
+        query = model_query(models.NodeGroup)
+        if not context.is_admin:
+            query = query.filter_by(project_id=context.project_id)
+        query = query.filter_by(cluster_id=cluster_id)
+        query = self._add_nodegoup_filters(query, filters)
+        return _paginate_query(models.NodeGroup, limit, marker,
+                               sort_key, sort_dir, query)
+
+    def get_cluster_nodegroup_count(self, context, cluster_id):
+        query = model_query(models.NodeGroup)
+        if not context.is_admin:
+            query = query.filter_by(project_id=context.project_id)
+        query = query.filter_by(cluster_id=cluster_id)
+        return query.count()
