@@ -1,12 +1,10 @@
-#!/bin/sh
-
 . /etc/sysconfig/heat-params
 
 step="enable-keystone-auth"
 printf "Starting to run ${step}\n"
 
 if [ "$(echo $KEYSTONE_AUTH_ENABLED | tr '[:upper:]' '[:lower:]')" != "false" ]; then
-    _prefix=${CONTAINER_INFRA_PREFIX:-docker.io/openstackmagnum/}
+    _prefix=${CONTAINER_INFRA_PREFIX:-docker.io/k8scloudprovider/}
     CERT_DIR=/etc/kubernetes/certs
 
     # Create policy configmap for keystone auth
@@ -66,6 +64,17 @@ metadata:
 data:
   policies: |
     $KEYSTONE_AUTH_DEFAULT_POLICY
+---
+apiVersion: v1
+kind: ConfigMap
+metadata:
+  name: keystone-sync-policy
+  namespace: kube-system
+data:
+  syncConfig: |
+    role-mappings:
+      - keystone-role: member
+        groups: []
 EOF
     }
 
@@ -77,7 +86,7 @@ EOF
         mkdir -p $(dirname ${KEYSTONE_AUTH_DEPLOY})
         cat << EOF > ${KEYSTONE_AUTH_DEPLOY}
 ---
-apiVersion: extensions/v1beta1
+apiVersion: apps/v1
 kind: DaemonSet
 metadata:
   labels:
@@ -87,6 +96,9 @@ metadata:
   namespace: kube-system
 spec:
   # The controllers can only have a single active instance.
+  selector:
+    matchLabels:
+      k8s-app: k8s-keystone-auth
   template:
     metadata:
       name: k8s-keystone-auth
@@ -120,6 +132,8 @@ spec:
             - k8s-keystone-auth-policy
             - --keystone-url
             - ${AUTH_URL}
+            - --sync-configmap-name
+            - keystone-sync-policy
             - --keystone-ca-file
             - /etc/kubernetes/ca-bundle.crt
             - --listen
@@ -152,7 +166,7 @@ spec:
 EOF
     }
 
-    until  [ "ok" = "$(curl --silent http://127.0.0.1:8080/healthz)" ]
+    until  [ "ok" = "$(kubectl get --raw='/healthz')" ]
     do
         echo "Waiting for Kubernetes API..."
         sleep 5
